#define MOZ_SOURCE_STAMP 3148731bed454d47fbd21518a308d0ec85458ccc
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-release
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-release/rev/3148731bed454d47fbd21518a308d0ec85458ccc
